////////////////////////////////////////////////////////////////////// 
// lamp/JointModel.h 
// (c) 2000-2006 Goncalo Abecasis
// 
// This file is distributed as part of the LAMP source code package   
// and may not be redistributed in any form, without prior written    
// permission from the author. Permission is granted for you to       
// modify this file for your own personal use, but modified versions  
// must retain this copyright notice and must not be distributed.     
// 
// Permission is granted for you to use this file to compile LAMP.    
// 
// All computer programs have bugs. Use this file at your own risk.   
// 
// Friday September 01, 2006
// 
 
#ifndef __JOINTMODEL_H__
#define __JOINTMODEL_H__

#include "OptimizerConstraints.h"
#include "JointLikelihood.h"
#include "ModelSummary.h"
#include "MarkerMap.h"
#include "Mantra.h"
#include "Houdini.h"

class GeneralMinimizer;

class JointModel : public ObjectiveFunction
   {
   public:
      // These static variables are used to customize analysis

      // This set of parameters is used to define limitations of the sample
      static bool   UNRELATEDS, SIBPAIRS, NOCONSTRAINTS, AUTOMATIC;

      // This first set of parameters is used to customize the disease model
      static double PREVALENCE;
      static bool   ADDITIVE, DOMINANT, RECESSIVE, MULTIPLICATIVE, FREE;

      // These parameters select an appropriate optimization algorithm
      static int    ROUNDS;
      static bool   NELDERMEAD, FLETCHERREEVES, STOCHASTIC;
      static double PRECISION;

      static int    MAXBITS;
      static bool   BUFFERING, SKIPCAUSAL;

      JointModel();
      virtual ~JointModel();

      bool CheckMaps(MarkerMap & frame, MarkerMap & snps);

      void  AllocateMemory(Pedigree & ped, MarkerMap & frame);
      void  FreeMemory();

      void  ScoreSinglepoint(Pedigree & ped, MarkerMap & frame);
      void  ScoreConditionals(Pedigree & ped, MarkerMap & frame);
      void  Analyze(Pedigree & ped, MarkerMap & frame, MarkerMap & candidates);
      void  AnalyzeQTL(Pedigree & ped, MarkerMap & frame, MarkerMap & candidates);

      virtual double Evaluate(Vector & point);

      static bool quiet;

   protected:
      GeneralMinimizer * GetMinimizer();

   private:
      Mantra * mantra;
      Tree  ** singlepoint;
      Tree  ** right;
      Tree  ** left;
      Tree   * linkage;
      Tree   * buffer;

      void  ScoreLinkage(Pedigree & ped, MarkerMap & frame, double position);

      int   families;
      int   currentMarker;
      int   currentAllele;
      int   currentModel;
      int   currentAffection;
      MarkerInfo * markerInfo;

      bool   mendelFlag;

      void  SetupConstraints(OptimizerInterface & optim, Vector & point);
      void  GetStartingPoint(Vector & point, int round = 0);
      void  ChooseStartingPoint(Vector & point);
      void  SetModelParameters(Vector & point);
      void  SelectModel(int model);

      bool  qtlMode;

      JointLikelihood lk;
      QtlLikelihood  qtl;

      double disease[2];

      IntArray bitsFilter;

      void MendelFilter(Pedigree & ped);
      void BuildBitsFilter(Pedigree & ped);

      void ScalePenetrances(double prevalence,
                            Vector & frequencies,
                            Vector & penetrances,
                            bool first_genotype_increases_risk = false);

      void ScaleMeans(Vector & frequencies, Vector & means, double & variance);

      void ScaleMeans(double frequency, double add, double dom,
                      Vector & means, double & residualVariance);

      ModelSummary details[4];

      void OpenDetailFiles();
      void CloseDetailFiles();
      void PrintDetails(int model, String & marker, String & trait, double llk, int parameters);

      void PrintLOD(double likelihoodRatio, int df);

      IntArray     leftScaling, rightScaling;
      double       adjustment;

      IntArray     lookup;

      void BuildLookup(Pedigree & ped);
   };

#endif


 
